class BookItError(Exception):
    def __repr__(self):
        return '{} ({})'.format(self.__class__.__name__, self.__dict__)


class BookItLoginFailedError(BookItError):
    def __init__(self, username):
        self.username = username



import traceback

import click

from studentit.bookit.api.api_client import ApiClient


@click.group()
@click.option('--username')
@click.option('--password')
@click.pass_context
def cli(ctx, username, password):
    ctx.obj = ApiClient(username, password)


@cli.group()
def server():
    pass


@cli.group()
def admin():
    pass


@cli.group()
def site():
    pass


@cli.group()
def resource():
    pass


@cli.group()
def booking():
    pass


@booking.command()
@click.argument('start_time')
@click.argument('end_time')
@click.argument('booking_date')
@click.argument('resource_id')
@click.option('--email-receipt', required=True, default=False)
@click.pass_obj
def create(client, start_time, end_time, booking_date, resource_id, email_receipt):
    print(client.create_booking(start_time, end_time, booking_date, resource_id, email_receipt))


@booking.command()
@click.argument('booking_id')
@click.option('--email-receipt', required=True, default=False)
@click.pass_obj
def delete(client, booking_id, email_receipt):
    print(client.delete_booking(booking_id, email_receipt))


@booking.command()
@click.pass_obj
def list(client):
    print(client.list_bookings())


@site.command(name='status')
@click.option('--site-id')
@click.pass_obj
def site_resource_status(client, site_id):
    print(client.all_resource_status(site_id=site_id))


@server.command(name='status')
@click.pass_obj
def server_status(client):
    if client.server_status():
        print('BookIT is up and responding to requests')
    else:
        print('BookIT is down')


@admin.command(name='all-resource-status')
@click.option('--site-id')
@click.pass_obj
def admin_all_resource_status(client, site_id):
    print(client.admin_all_resource_status(site_id))


@admin.command(name='location-status')
@click.option('--location-id', required=True)
@click.pass_obj
def admin_location_status(client, location_id):
    print(client.admin_location_status(location_id))


def main():
    try:
        cli(auto_envvar_prefix='BOOKIT')
    except Exception as e:
        print('ERROR: ' + str(e))
        traceback.print_exc()


if __name__ == '__main__':
    main()



import logging
import re
import time

import requests

from studentit.bookit.api.exceptions import BookItLoginFailedError


class ApiAdapter(object):
    def __init__(self, username, password):
        self.username = username
        self.password = password

        self.logger = logging.getLogger(__name__)

        self._session = requests.Session()

        if self.username and self.password:
            self._do_login()

    def _do_login(self):
        login_string = '<Login><username>{}</username><password>{}</password><rememberMe>false</rememberMe>' \
                       '<rememberView>-</rememberView></Login>'.format(self.username, self.password)
        login_res = self.post(endpoint='cire/login.aspx', data=login_string)
        match = re.search("\'http://bookit.unimelb.edu.au/(.*)\'", login_res.text)

        if not match or not self.get(endpoint=match.group(1)).status_code == 200:
            raise BookItLoginFailedError(username=self.username)

    def post(self, endpoint, *args, **kwargs):
        return self._request(method=self._session.post, endpoint=endpoint, *args, **kwargs)

    def get(self, endpoint, *args, **kwargs):
        res = self._request(method=self._session.get, endpoint=endpoint, *args, **kwargs)
        wait = 0.25
        cur_try = 0
        max_tries = 5
        while 'exception' in res.text.lower() and cur_try < max_tries:
            cur_try += 1
            self.logger.error(
                'Exception in API call to {endpoint}. Retrying [{cur_try}/{max_tries}]'.format(endpoint=endpoint,
                                                                                               cur_try=cur_try,
                                                                                               max_tries=max_tries))
            time.sleep(wait * cur_try)
            res = self._request(method=self._session.get, endpoint=endpoint, *args, **kwargs)
        return res

    def _request(self, endpoint, method, *args, **kwargs):
        res = method(self._get_url(endpoint), *args, **kwargs)

        if "location.href = 'http://834S-MYPC/cire/login.aspx'" in res.text:
            self.logger.error('Session expired. Reauthenticating...')
            self._do_login()
            res = method(self._get_url(endpoint), *args, **kwargs)

        return res

    def _get_url(self, endpoint):
        return 'https://bookit.unimelb.edu.au/{}'.format(endpoint)
